// $Id: settings.h,v 1.4 2002/04/22 23:07:41 jjlupa Exp $
//
// ypost (Yet Another Poster)
// This code is copyright John Ridley, 2000. <john.ridley@gt.org>  To the
//  extent that it is released, it is released under the GNU General Public License
//  v2 or greater. The author retains all rights with respect to this code.
//
// If you have a desire to use this code outside of the GPL, drop me an email and
//  I would be happy to work with you under an alternate licence.  If you just want
//  to write code, then I would probably be happy to release it to you as under
//  the perl artistic license.

// settings.h
// Global object that contains user's default and selected options

#ifndef YPOST_SETTINGS_H_
#define YPOST_SETTINGS_H_

#include <string>
#include <map>
#include <vector>

class userSettings; // forward reference

class FileObj
{
public:
    friend class userSettings;

    // either text or binary; if binary, use global encode method.
    typedef enum {_text_, _bin_uu_, _bin_yenc_} encodeMethod;

    FileObj(const char *pszFilename, encodeMethod method);
    // copy constructor
    //  FileObj(const FileObj &that);

    // do copy operator

    bool postThisPart(int part);
    std::string getFilename()
    {
        return m_filename;
    }
    encodeMethod getEncodeMethod()
    {
        return m_method;
    }
    long getBytes()
    {
        return m_bytes;
    }
    void setBytes(long bytes)
    {
        m_bytes = bytes;
    }

protected:
    std::string  m_filename;
    encodeMethod m_method;
    std::vector<int>  m_partsToPost;  // all if empty
    long m_bytes; // just for cosmetics, don't count on it being there for now
};


// singleton
class userSettings
{
public:
    // singleton access
    static userSettings &obj();
    userSettings();

    void printUsage();

    // setup
    void parseArgs(int argc, char *argv[]);
    void readSettingsFile(const char *filename = 0);
    void writeSettingsFile(const char *filename = 0);

    // data access
    bool defined(const char *pszKey);
    std::string getValue(const char *pszKey);
    std::string getValue( const char * pszKey, const char * def );

    int numFiles();
    FileObj &getFileObj(int idx);
    int verbosity;

    // filelist iterator
protected:
    void defaults();
    void printVersion();
    void printJobSummary();
    bool validateArgs();
    bool parseparts(const char *filearg, FileObj &fo);

    int parseOneArg(int argc, char *argv[]);
    void recordExtraArg(int argc, char *argv[],
                        const char *cmdname,
                        bool xheaders=false);

    // uu or yenc, this doesn't apply to text encoding
    FileObj::encodeMethod m_binEncoding;
    bool m_invalidArgFound, m_writeDefaults;
    std::vector<FileObj> m_files;
    std::map<std::string,std::string> m_values;
};

#endif // _YPOST_SETTINGS_H_
