// $Id: uuencode.cpp,v 1.4 2002/04/22 23:07:41 jjlupa Exp $
//
// ypost (Yet Another Poster)
// This code is copyright John Ridley, 2000. <john.ridley@gt.org>  To the
//  extent that it is released, it is released under the GNU General Public License
//  v2 or greater. The author retains all rights with respect to this code.
//
// If you have a desire to use this code outside of the GPL, drop me an email and
//  I would be happy to work with you under an alternate licence.  If you just want
//  to write code, then I would probably be happy to release it to you as under
//  the perl artistic license.

namespace std
    {}
using namespace std;

#ifdef WIN32 // God can this get annoying...
#   pragma warning(disable:4786)
#endif

#include "uuencode.h"

#ifdef WIN32
#   pragma warning(disable:4786)
#else
#   include <stdio.h>
#   include <unistd.h>
#endif

// *********************************************************
// sample usage:
//  uuencode coder("/home/jrr/work/18ktest",400);
//  long NumSegments = coder.NumSegments();
//  long BufSize = coder.BufSpaceNeeded();
//  char *buf = (char *)malloc(BufSize);
//  for (int x=0; x<NumSegments; x++)
//  {
//    if (!coder.EncodeSegment(x+1,buf,BufSize))
//    {
//      printf( "Unable to encode segment %d; quitting\n", x+1);
//      exit(255);
//    }
//    char filename[128];
//    sprintf(filename,"file.uu.%3.3d",x+1);
//    FILE *file = fopen(filename,"wb");
//    fputs(buf, file);
//    fclose(file);
//  }
// ********************************************************

uuencode::uuencode(const char *pszFilename,
                   long LinesPerPost) :
        generic_encoder(pszFilename, LinesPerPost)
{
    // calc num segments for this encode type and save it off.
    long BytesPerSegment = m_LinesPerPost * _RawBytesPerLine;
    m_numSegments = m_filesize / BytesPerSegment;
    if ( (m_filesize % BytesPerSegment) > 0)
        m_numSegments++;
}


long uuencode::BufSpaceNeeded()
{
    return (m_LinesPerPost+1) * (_EncodedBytesPerLine+2);
}

// all or nothing copy to buffer
static long CopyStr(const char *pszFrom, char *ToBuf, long size)
{
    long SrcLen = strlen(pszFrom);
    if (SrcLen > size)
    {
        return false;
    }
    strcpy(const_cast<char *>(pszFrom), ToBuf);
    return true;
}

static char *BufCat(const char *pszFrom, char *ToBuf)
{
    while (*pszFrom)
    {
        *(ToBuf++) = *(pszFrom++);
    }
    *ToBuf = 0;
    return ToBuf;
}


static const char uu_std[64] =
    {
        '`', '!', '"', '#', '$', '%', '&', '\'',
        '(', ')', '*', '+', ',', '-', '.', '/',
        '0', '1', '2', '3', '4', '5', '6', '7',
        '8', '9', ':', ';', '<', '=', '>', '?',
        '@', 'A', 'B', 'C', 'D', 'E', 'F', 'G',
        'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O',
        'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W',
        'X', 'Y', 'Z', '[', '\\', ']', '^', '_'
    };
// ENC is the basic 1 character encoding function to make a char printing.
#define ENC(Char) (uu_std[(Char) & 077])


// zero terminated whole lines to buffer
// portions of this code adapted from FSF's uuencode utility
bool uuencode::EncodeSegment(long segment, char *buf, long size)
{
    long CurPos, StartPos, EndPos;
    std::string line;
    bool terminalSegment(false);

    // sanity checks
    if (   (size < BufSpaceNeeded())
            || (segment < 1)
            || (segment > m_numSegments))
        return false;

    if (1 == segment)  // do header for 1st segment
    {
        // begin 644 uuencode.cpp
        // begin mode filename - we'll just use mode 644 for everything.
        std::string filename;
        // get just filename part (no path)
        int pos = m_filename.find_last_of('/');
        if (-1 != pos)
        {
            filename = m_filename.substr(pos+1);
        }
        else
        {
            filename = m_filename;
        }

        buf = BufCat("begin 644 ", buf);
        buf = BufCat(filename.c_str(), buf);
        buf = BufCat("\n", buf);
    }

    // calc start position (input file pos)
    StartPos = (segment - 1) * m_LinesPerPost * _RawBytesPerLine;
    CurPos = StartPos;

    // calc terminal position for this segment
    EndPos = StartPos + (m_LinesPerPost * _RawBytesPerLine) - 1;
    long FileSize = m_filesize - 1; // convert to zero based for later calcs
    if (EndPos >= FileSize)
    {
        terminalSegment = true;
        EndPos = FileSize;
    }

    // open the file
    FILE *infile;
    infile = fopen(m_filename.c_str(),"rb");
    if (NULL == infile)
        return false;
    fseek(infile,StartPos,SEEK_SET);

    // now the encoding starts.  45 bytes per line
    while (CurPos <= EndPos)
    {
        unsigned char inputbuf[45];

        // calc bytes this line
        long BytesThisLine = EndPos - CurPos + 1;
        if (BytesThisLine > _RawBytesPerLine)
            BytesThisLine = _RawBytesPerLine;
        *(buf++) = ENC(BytesThisLine);
        CurPos += BytesThisLine;

        // read the line's raw data
        memset(inputbuf,0,45); // clear buffer
        int n,m;
        n=0;
        m=0;
        do
        {
            m = fread (inputbuf, 1, 45 - n, infile);
            if (m == 0)
                break;
            n += m;
        }
        while (n < BytesThisLine);

        // take 3 byte chunks, expand the bits into 4 bytes
        // then encode those 4 bytes
        for (unsigned char *p = inputbuf;
                BytesThisLine > 0;
                BytesThisLine -= 3, p += 3)
        {
            unsigned char ch;
            ch = *p >> 2;
            *(buf++) = ENC (ch);
            ch = ((*p << 4) & 060) | ((p[1] >> 4) & 017);
            *(buf++) = ENC (ch);
            ch = ((p[1] << 2) & 074) | ((p[2] >> 6) & 03);
            *(buf++) = ENC (ch);
            ch = p[2] & 077;
            *(buf++) = ENC (ch);
        }
        *(buf++) = '\n';
    }
    if (terminalSegment)
    {
        strcpy(buf,"`\nend\n");
    }
    else
    {
        *buf = 0;
    }
    fclose(infile);
    return true;
}
