// $Id: win32.cpp,v 1.2 2002/04/22 23:07:41 jjlupa Exp $
//
// ypost (Yet Another Poster)
// This code is copyright Jonathan Lupa, 2000. <jjlupa@jamdata.net>  To the
//  extent that it is released, it is released under the GNU General Public License
//  v2 or greater. The author retains all rights with respect to this code.
//
// If you have a desire to use this code outside of the GPL, drop me an email and
//  I would be happy to work with you under an alternate licence.  If you just want
//  to write code, then I would probably be happy to release it to you as under
//  the perl artistic license.
//
// Implements independent.h

#ifdef WIN32

#pragma warning(disable:4786)

#include "independent.h"

#include <windows.h>
#include <assert.h>

#include <string>
#include <iostream>

using namespace std;

namespace independent
{
    void sleep( unsigned int milli_seconds )
    {
        ::Sleep( milli_seconds );
    }

    char slash()
    {
        return '\\';
    }

    int sock_error()
    {
        return ::WSAGetLastError();
    }

    void get_time( timeval & tv )
    {
        const DWORD ticks = ::GetTickCount();
        tv.tv_sec = ticks / 1000;
        tv.tv_usec = ( ticks % 1000 ) * 1000;
    }
}

namespace win32
{
    void expand_fns( std::string strmask, std::vector<std::string> & filenames )
    {
        assert( strmask.size() && filenames.size() == 0 );

        if ( strmask.find( '*' ) == string::npos )
        {
            filenames.push_back( strmask );
        }
        else
        {
            WIN32_FIND_DATA fd;
            memset( &fd, 0, sizeof( fd ) );

            // We actually have a mask... lets do this thing.
            HANDLE hfind = ::FindFirstFile( strmask.c_str(), &fd );
            if ( hfind == INVALID_HANDLE_VALUE )
            {
                assert( false );
                cerr << "Failed to find any files matching " << strmask << " err=" << ::GetLastError() << endl;
                return;
            }

            filenames.push_back( fd.cFileName );

            while ( ::FindNextFile( hfind, &fd ) )
            {
                filenames.push_back( fd.cFileName );
            }

            if ( ::GetLastError() != ERROR_NO_MORE_FILES )
                cerr << "Unexpected error while expanding wildcards... err=" << ::GetLastError() << endl;

            ::FindClose( hfind );

        }
    }

    wsa::wsa()
    {
        const WORD wVersion = MAKEWORD( 1, 1 );
        WSADATA wsadata;
        memset( &wsadata, 0, sizeof( wsadata ) );
        int ret = ::WSAStartup( wVersion, &wsadata );
        if ( ret != 0 )
            cerr << "Failed to initialize WinSock... err=" << ret << endl;
    }
    wsa::~wsa()
    {
        ::WSACleanup();
    }


}

#endif
