// $Id: ypost.cpp,v 1.6 2002/04/26 15:18:13 jjlupa Exp $
//
// ypost (Yet Another Poster)
// This code is copyright John Ridley, 2000. <john.ridley@gt.org>  To the
//  extent that it is released, it is released under the GNU General Public License
//  v2 or greater. The author retains all rights with respect to this code.
//
// If you have a desire to use this code outside of the GPL, drop me an email and
//  I would be happy to work with you under an alternate licence.  If you just want
//  to write code, then I would probably be happy to release it to you as under
//  the perl artistic license.

// Main program body

// define to not really post
// #define _dummy_

namespace std
    {}
using namespace std;

#ifdef WIN32 // God can this get annoying...
#   pragma warning(disable:4786)
#endif

#include "ypost.h"
#include "settings.h"
#include "uuencode.h"
#include "yenc.h"
#include "textencoder.h"
#include "socket.h"
#include "nntp.h"
#include "independent.h"
#include "common.h"

#include <string>
#include <iostream>

#ifdef WIN32
#   include <time.h>
#   include <winsock2.h>
#else
#   include <stdio.h>
#   include <unistd.h>
#   include <sys/types.h>
#   include <sys/stat.h>
#   include <sys/time.h>
#endif

// global bytes posted
long bytesPosted;

Ypost_Exception::Ypost_Exception(extype severity, std::string info) :
        m_Severity(severity),
        m_Info(info)
{}

//
// postFile() : 
//
void PostFile(FileObj &file, nntp &nntpposter)
{
    std::string filename = file.getFilename();

    generic_encoder *pCoder;
    long linesperpost = atol(userSettings::obj().getValue("lines").c_str());
    if (file.getEncodeMethod() == FileObj::_text_)
    {
        pCoder = new textencoder(filename.c_str(), linesperpost);
    }
    else if (file.getEncodeMethod() == FileObj::_bin_uu_)
    {
        pCoder = new uuencode(filename.c_str(), linesperpost);
    }
    else if (file.getEncodeMethod() == FileObj::_bin_yenc_)
    {
        pCoder = new yenc(filename.c_str(), linesperpost);
    }
    if (NULL == pCoder)
        throw Ypost_Exception(Ypost_Exception::_FATAL_,
                              std::string("Unable to create encoding object"));

    long NumSegments = pCoder->numSegments();
    long BufSize = pCoder->BufSpaceNeeded();
    char *buf = (char *)malloc(BufSize);
    for (int segment=1; segment <= NumSegments; segment++)
    {
        if (!file.postThisPart(segment))
            continue;

        if (!pCoder->EncodeSegment(segment, buf, BufSize))
        {
            printf( "Unable to encode segment %d; quitting\n", segment);
            exit(255);
        }
        // TODO: convert meta strings in provided string
        char subject[512];

        if (file.getEncodeMethod() == FileObj::_text_)
        {
            sprintf(subject,"%s",
                    userSettings::obj().getValue("subject").c_str());
        }
        else
        {
            // put the yEnc first.  It's not per spec, but it's the RIGHT
            // thing to do.
            sprintf(subject,"%s - %s\"%s\" (%2.2d/%2.2d)",
                    userSettings::obj().getValue("subject").c_str(),
                    (file.getEncodeMethod() == FileObj::_bin_yenc_
                     ? "yEnc " : ""),
                    filename.c_str(),
                    segment, NumSegments);
        }
        if (userSettings::obj().verbosity > 0)
            cerr << "Posting \"" << subject << "\"" << endl;

        bool TryAgain;

	const int MaxRetries = atoi( userSettings::obj().getValue("max_retries","60").c_str() );
        int retries(MaxRetries);
        do
        {
            TryAgain = false;
            try
            {
                if (!nntpposter.postArticle(
                            userSettings::obj().getValue("newsgroups").c_str(),
                            userSettings::obj().getValue("from").c_str(),
                            subject,
                            userSettings::obj().getValue("xheaders").c_str(),
                            buf))
                {
                    throw Ypost_Exception(Ypost_Exception::_NONFATAL_,nntpposter.lastResult());
                }
                bytesPosted += nntpposter.bytesPosted();
            }
            catch (Ypost_Exception ex)
            {
                cerr << "Caught exception " << ex.getInfo() << endl;
                if (ex.getSeverity() != Ypost_Exception::_FATAL_)
                {
                    if (--retries <= 0)
                    {
                        cerr << "Too many retries, I give up." << endl;
                        throw ex;
                    }
                    else
                    {
                        cerr << "Nonfatal; we'll try again" << endl;
			const int wait = atoi( userSettings::obj().getValue( "retry_wait", "240" ).c_str() );
                        independent::sleep( wait * 1000 ); // about 4 or 5 minutes is probably good...
                        TryAgain = true;
                    }
                }
                else
                {
                    cerr << "Fatal, passing exception on up" << endl;
                    throw ex;
                }
            }
        }
        while (TryAgain);

    }
    free(buf);
}


int main(int argc, char *argv[])
{
    try
    {
#ifdef WIN32
        win32::wsa _wsa;
#endif

        userSettings::obj().parseArgs(argc, argv);
        int NumFiles = userSettings::obj().numFiles();

        if (userSettings::obj().verbosity > 0)
        {
            const int wait = atoi(userSettings::obj().getValue("post_wait","10").c_str());
            cerr << "Posting will begin in " << wait << " seconds..." << endl;
            independent::sleep( wait * 1000 ); //10s
        }

        // mark start time

        timeval start_tv;
        independent::get_time(start_tv);

        int port = atoi(userSettings::obj().getValue("port").c_str());
        nntp nntpposter(userSettings::obj().getValue("hostname").c_str(), port);
        // set auth info if given
        if (userSettings::obj().defined("userid"))
        {
            nntpposter.setAuthInfo(
                userSettings::obj().getValue("userid").c_str(),
                userSettings::obj().getValue("password").c_str());
        }
        int filenum;
        for (filenum=0; filenum < NumFiles; filenum++ )
        {
            PostFile(userSettings::obj().getFileObj(filenum), nntpposter);
        }
        if (userSettings::obj().verbosity > 0)
        {

            timeval end_tv;
            independent::get_time(end_tv);
            const double elapsedSeconds = common::timeval_diff( end_tv, start_tv ) / 1000000.0;

	    if ( userSettings::obj().verbosity > 5 )
	    {
		cerr << "TIME: START[" << start_tv.tv_sec << "," << start_tv.tv_usec << "], END[" 
		     << end_tv.tv_sec << "," << end_tv.tv_usec << "], ELAPSED_USEC[" << common::timeval_diff( end_tv, start_tv )
		     << "],  ELAPSED_S[" << elapsedSeconds << "]" << endl;
	    }
            const long bps = static_cast<long>( bytesPosted / elapsedSeconds );

            cerr << "Summary:" << endl
            << bytesPosted << " bytes posted in " << elapsedSeconds << " seconds" << endl
            << "Average throughput: " << bps << " bytes per second (this value includes all response times)." << endl;
        }
    }
    catch(Ypost_Exception ex)
    {
        cerr << ex.getInfo() << endl;
        exit(255);
    }

    return 0;
}
